/**
 * Copyright (C) 2011-2012 enStratus Networks Inc
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.cloudsigma;

import static org.jclouds.concurrent.MoreExecutors.sameThreadExecutor;

import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudException;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.jclouds.cloudsigma.compute.CSComputeServices;
import org.dasein.cloud.jclouds.cloudsigma.network.CSNetworkServices;
import org.jclouds.cloudsigma.CloudSigmaAsyncClient;
import org.jclouds.cloudsigma.CloudSigmaClient;
import org.jclouds.compute.ComputeServiceContextFactory;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;
import org.jclouds.rest.RestContext;

import com.google.inject.Module;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;


public class CloudSigma extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(CloudSigma.class);
    
    public @Nonnull String getCloudName() {
        return "CloudSigma";
    }

    public @Nonnull String getProviderName() {
        return "CloudSigma";
    }

    public @Nonnull RestContext<CloudSigmaClient,CloudSigmaAsyncClient> getCloudClient() throws CloudException {
        ProviderContext ctx = getContext();
        
        if( ctx == null ) {
            throw new CloudException("No context was defined for this request");
        }
        String user = new String(ctx.getAccessPublic());
        String key = new String(ctx.getAccessPrivate());
        
        String endpoint = ctx.getEndpoint();
        
        ArrayList<Module> modules = new ArrayList<Module>();
        
        modules.add(new Log4JLoggingModule());
        modules.add(new ExecutorServiceModule(sameThreadExecutor(), sameThreadExecutor()));
        if( endpoint == null || endpoint.trim().equals("") || endpoint.startsWith("https://api.cloudsigma.com") || endpoint.startsWith("https://api.zrh.cloudsigma.com") ) {
        	return (new ComputeServiceContextFactory()).createContext("cloudsigma-zrh", user, key, modules).getProviderSpecificContext();
        }
        else if( endpoint.startsWith("https://api.lvs.cloudsigma.com") ){
        	return (new ComputeServiceContextFactory()).createContext("cloudsigma-lvs", user, key,modules).getProviderSpecificContext();              
        }
        else {
            throw new CloudException("Dasein Cloud does not yet support the endpoint: " + endpoint);
        }
    }
    
    @Override
    public @Nonnull CSComputeServices getComputeServices() {
        return new CSComputeServices(this);
    }
    
    @Override
    public @Nonnull CSDC getDataCenterServices() {
        return new CSDC(this);
    }
    
    @Override
    public @Nonnull CSNetworkServices getNetworkServices() {
        return new CSNetworkServices(this);
    }
    
    @Override
    public @Nullable String testContext() {
        try {
            RestContext<CloudSigmaClient,CloudSigmaAsyncClient> ctx = getCloudClient();

            try {
                return ctx.getApi().getProfileInfo().getUuid();
            }
            finally {
                ctx.close();
            }
        }
        catch( Throwable t ) {
            ProviderContext ctx = getContext();

            logger.warn("Unable to test CloudSigma context for " + (ctx == null ? "null" : ctx.getAccountNumber()) + ": " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
